/*----- PROTECTED REGION ID(E2SextupoleClass.cpp) ENABLED START -----*/
/* clang-format on */
//=============================================================================
//
// file :        E2SextupoleClass.cpp
//
// description : C++ source for the E2SextupoleClass.
//               A singleton class derived from DeviceClass.
//               It implements the command and attribute list
//               and all properties and methods required
//               by the E2Sextupole once per process.
//
// project :     Elettra 2.0 pure corrector magnet device
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
//
// Elettra - Sincrotrone Trieste S.c.p.A.
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include "E2SextupoleClass.h"
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass.cpp

//-------------------------------------------------------------------
/**
 *	Create E2SextupoleClass singleton and
 *	return it in a C function for Python usage
 */
//-------------------------------------------------------------------
extern "C" {
#ifdef _TG_WINDOWS_

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_E2Sextupole_class(const char *name) {
		return E2Sextupole_ns::E2SextupoleClass::init(name);
	}
}

namespace E2Sextupole_ns
{
//===================================================================
//	Initialize pointer for singleton pattern
//===================================================================
E2SextupoleClass *E2SextupoleClass::_instance = NULL;

//===================================================================
//	Class constants
//===================================================================
constexpr long PolynomAAttrib::X_DATA_SIZE;
constexpr long PolynomBAttrib::X_DATA_SIZE;
//--------------------------------------------------------
/**
 * method : 		E2SextupoleClass::E2SextupoleClass(std::string &s)
 * description : 	constructor for the E2SextupoleClass
 *
 * @param s	The class name
 */
//--------------------------------------------------------
E2SextupoleClass::E2SextupoleClass(std::string &s):Tango::DeviceClass(s)
{
	TANGO_LOG_INFO << "Entering E2SextupoleClass constructor" << std::endl;
	set_default_property();
	get_class_property();
	write_class_property();

	/*----- PROTECTED REGION ID(E2SextupoleClass::constructor) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::constructor

	TANGO_LOG_INFO << "Leaving E2SextupoleClass constructor" << std::endl;
}

//--------------------------------------------------------
/**
 * method : 		E2SextupoleClass::~E2SextupoleClass()
 * description : 	destructor for the E2SextupoleClass
 */
//--------------------------------------------------------
E2SextupoleClass::~E2SextupoleClass()
{
	/*----- PROTECTED REGION ID(E2SextupoleClass::destructor) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::destructor

	_instance = NULL;
}


//--------------------------------------------------------
/**
 * method : 		E2SextupoleClass::init
 * description : 	Create the object if not already done.
 *                  Otherwise, just return a pointer to the object
 *
 * @param	name	The class name
 */
//--------------------------------------------------------
E2SextupoleClass *E2SextupoleClass::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			std::string s(name);
			_instance = new E2SextupoleClass(s);
		}
		catch (std::bad_alloc &)
		{
			throw;
		}
	}
	return _instance;
}

//--------------------------------------------------------
/**
 * method : 		E2SextupoleClass::instance
 * description : 	Check if object already created,
 *                  and return a pointer to the object
 */
//--------------------------------------------------------
E2SextupoleClass *E2SextupoleClass::instance()
{
	if (_instance == NULL)
	{
		std::cerr << "Class is not initialized !!" << std::endl;
		exit(-1);
	}
	return _instance;
}



//===================================================================
//	Command execution method calls
//===================================================================
//--------------------------------------------------------
/**
 * method : 		ResetClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ResetClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	TANGO_LOG_INFO << "ResetClass::execute(): arrived" << std::endl;
	((static_cast<E2Sextupole *>(device))->reset());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		ResetResonanceStrengthClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ResetResonanceStrengthClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	TANGO_LOG_INFO << "ResetResonanceStrengthClass::execute(): arrived" << std::endl;
	((static_cast<E2Sextupole *>(device))->reset_resonance_strength());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		ApplyDesignStrengthClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ApplyDesignStrengthClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	TANGO_LOG_INFO << "ApplyDesignStrengthClass::execute(): arrived" << std::endl;
	((static_cast<E2Sextupole *>(device))->apply_design_strength());
	return new CORBA::Any();
}


//===================================================================
//	Properties management
//===================================================================
//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::get_class_property()
 *	Description: Get the class property for specified name.
 */
//--------------------------------------------------------
Tango::DbDatum E2SextupoleClass::get_class_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, returns  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::get_default_device_property()
 *	Description: Return the default value for device property.
 */
//--------------------------------------------------------
Tango::DbDatum E2SextupoleClass::get_default_device_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::get_default_class_property()
 *	Description: Return the default value for class property.
 */
//--------------------------------------------------------
Tango::DbDatum E2SextupoleClass::get_default_class_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::get_class_property()
 *	Description: Read database to initialize class property data members.
 */
//--------------------------------------------------------
void E2SextupoleClass::get_class_property()
{
	/*----- PROTECTED REGION ID(E2SextupoleClass::get_class_property_before) ENABLED START -----*/
	/* clang-format on */
	//	Initialize class property data members
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::get_class_property_before
	//	Read class properties from database.
	cl_prop.push_back(Tango::DbDatum("BbaSwitchDeviceName"));
	cl_prop.push_back(Tango::DbDatum("BeamEnergy"));
	cl_prop.push_back(Tango::DbDatum("CalibrationPath"));

	//	Call database and extract values
	if (Tango::Util::instance()->_UseDb==true)
		get_db_class()->get_property(cl_prop);
	Tango::DbDatum	def_prop;
	int	i = -1;

	//	Try to extract BbaSwitchDeviceName value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  bbaSwitchDeviceName;
	else
	{
		//	Check default value for BbaSwitchDeviceName
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop    >>  bbaSwitchDeviceName;
			cl_prop[i]  <<  bbaSwitchDeviceName;
		}
	}
	//	Try to extract BeamEnergy value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  beamEnergy;
	else
	{
		//	Check default value for BeamEnergy
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop    >>  beamEnergy;
			cl_prop[i]  <<  beamEnergy;
		}
	}
	//	Try to extract CalibrationPath value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  calibrationPath;
	else
	{
		//	Check default value for CalibrationPath
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop    >>  calibrationPath;
			cl_prop[i]  <<  calibrationPath;
		}
	}
	/*----- PROTECTED REGION ID(E2SextupoleClass::get_class_property_after) ENABLED START -----*/
	/* clang-format on */
	//	Check class property data members init
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::get_class_property_after

}

//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::set_default_property()
 *	Description: Set default property (class and device) for wizard.
 *                For each property, add to wizard property name and description.
 *                If default value has been set, add it to wizard property and
 *                store it in a DbDatum.
 */
//--------------------------------------------------------
void E2SextupoleClass::set_default_property()
{
	std::string	prop_name;
	std::string	prop_desc;
	std::string	prop_def;
	std::vector<std::string>	vect_data;

	//	Set Default Class Properties
	prop_name = "BbaSwitchDeviceName";
	prop_desc = "Tango device name exporting the interface to the BBA switches.\nPart of Elettra 2.0 MPS (Interlock) system";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);
	prop_name = "BeamEnergy";
	prop_desc = "default electron beam energy, in GeV.\nUse to initialize all magnet istances BeamEnergy attribute.";
	prop_def  = "2.4";
	vect_data.clear();
	vect_data.push_back("2.4");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);
	prop_name = "CalibrationPath";
	prop_desc = "path of calibration and parameter files";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);

	//	Set Default device Properties
	prop_name = "BbaSwitchAttribute";
	prop_desc = "Attribute name of the speciific ba switch\nThe BBA switches are part of Elettra 2.0 MPS (Interlock) system";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "BbaSwitchDeviceName";
	prop_desc = "Tango device name exporting the interface to the BBA switches.\nPart of Elettra 2.0 MPS (Interlock) system";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CEHCalibrationFileName";
	prop_desc = "name of calibration table file name for horizontal embedded corrector - to be review3d when real calibrtaion tablles wil be available";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CEHDeviceName";
	prop_desc = "Name of tango device implementing the interface of the embedded horizontal corrector.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CEHName";
	prop_desc = "Magnet name of embbedded horizontal corrector.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CEVCalibrationFileName";
	prop_desc = "name of calibration table file name for vertical embedded corrector - to be reviewed when real calibration tables wil be available";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CEVDeviceName";
	prop_desc = "Name of tango device implementing the interface of the embedded vertical corrector.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CEVName";
	prop_desc = "Magnet name of embbedded vertical corrector.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CalibrationFileName";
	prop_desc = "name of calibration table file name";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CalibrationPath";
	prop_desc = "path of calibration and parameter files";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CurrentDevice";
	prop_desc = "power supply tango device name";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "EmbeddedConfiguration";
	prop_desc = "String descibing the configuration fo the embedded coils. Only three values are valid:\n\npure_sext\nceh_cev\nsext_with_skew\nbba_ready\n\n\nDepending on the EbenddedConfiguration the appropriate numebr of proxies for power supplies and calibration models is built.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "MagnetName";
	prop_desc = "Name of magnet";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Model";
	prop_desc = "corrector model (part number)";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "ParameterFileName";
	prop_desc = "name of auxiliary parametr file name\n(EBSmagnet compatibility)";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "PsCehDeviceName";
	prop_desc = "Name of the tango device of powering the embbedded horizontal corrector / embedded skew quadrupole.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "PsCevDeviceName";
	prop_desc = "Name of the tango device of powering the embbedded vertical corrector.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "SQCalibrationFileName";
	prop_desc = "name of calibration table file name for embedded skeq quadrupole - to be reviewed when real calibration tables will be available";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "SerialNumber";
	prop_desc = "serial number of the magnet";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "SkewQuad";
	prop_desc = "True if the magnet is cabled with embbedd skew quadrupole QS . In this configuration the emmbedd horizontal corrector CEH is not available.";
	prop_def  = "False";
	vect_data.clear();
	vect_data.push_back("False");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
}

//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::write_class_property()
 *	Description: Set class description fields as property in database
 */
//--------------------------------------------------------
void E2SextupoleClass::write_class_property()
{
	//	First time, check if database used
	if (Tango::Util::_UseDb == false)
		return;

	Tango::DbData	data;
	std::string	classname = get_name();
	std::string	header;

	//	Put title
	Tango::DbDatum	title("ProjectTitle");
	std::string	str_title("Elettra 2.0 pure corrector magnet device");
	title << str_title;
	data.push_back(title);

	//	Put Description
	Tango::DbDatum	description("Description");
	std::vector<std::string>	str_desc;
	str_desc.push_back("Elettra 2.0 sextupole magnet device");
	str_desc.push_back("Use calibration table handle current/strgenght converstions.");
	str_desc.push_back("The magnet device is connected to a power supply tango device , real or simulated.");
	str_desc.push_back("");
	str_desc.push_back("Based on ESRF EBSmagnet concepts and libraries.");
	description << str_desc;
	data.push_back(description);

	//  Put inheritance
	Tango::DbDatum	inher_datum("InheritedFrom");
	std::vector<std::string> inheritance;
	inheritance.push_back("TANGO_BASE_CLASS");
	inher_datum << inheritance;
	data.push_back(inher_datum);

	//	Call database and and values
	get_db_class()->put_property(data);
}

//===================================================================
//	Factory methods
//===================================================================

//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::device_factory()
 *	Description: Create the device object(s)
 *                and store them in the device list
 */
//--------------------------------------------------------
void E2SextupoleClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{
	/*----- PROTECTED REGION ID(E2SextupoleClass::device_factory_before) ENABLED START -----*/
/* clang-format on */
//	Add your own code
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::device_factory_before

	//	Create devices and add it into the device list
	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		TANGO_LOG_DEBUG << "Device name : " << (*devlist_ptr)[i].in() << std::endl;
		device_list.push_back(new E2Sextupole(this, (*devlist_ptr)[i]));
	}

	//	Manage dynamic attributes if any
	erase_dynamic_attributes(devlist_ptr, get_class_attr()->get_attr_list());

	//	Export devices to the outside world
	for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++)
	{
		//	Add dynamic attributes if any
		E2Sextupole *dev = static_cast<E2Sextupole *>(device_list[device_list.size()-i]);
		dev->add_dynamic_attributes();

		//	Check before if database used.
		if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
			export_device(dev);
		else
			export_device(dev, dev->get_name().c_str());
	}

	/*----- PROTECTED REGION ID(E2SextupoleClass::device_factory_after) ENABLED START -----*/
/* clang-format on */
//	Add your own code
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::device_factory_after
}
//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::attribute_factory()
 *	Description: Create the attribute object(s)
 *                and store them in the attribute list
 */
//--------------------------------------------------------
void E2SextupoleClass::attribute_factory(std::vector<Tango::Attr *> &att_list)
{
	/*----- PROTECTED REGION ID(E2SextupoleClass::attribute_factory_before) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::attribute_factory_before
	//	Attribute : Strength
	StrengthAttrib	*strength = new StrengthAttrib();
	Tango::UserDefaultAttrProp	strength_prop;
	strength_prop.set_description("sextupole strength 1/m^3 ( m-3)");
	strength_prop.set_label("strength");
	strength_prop.set_unit("m-3");
	//	standard_unit	not set for Strength
	//	display_unit	not set for Strength
	//	format	not set for Strength
	//	max_value	not set for Strength
	//	min_value	not set for Strength
	//	max_alarm	not set for Strength
	//	min_alarm	not set for Strength
	//	max_warning	not set for Strength
	//	min_warning	not set for Strength
	//	delta_t	not set for Strength
	//	delta_val	not set for Strength
	strength->set_default_properties(strength_prop);
	//	Not Polled
	strength->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(strength);

	//	Attribute : CorrectionStrength
	CorrectionStrengthAttrib	*correctionstrength = new CorrectionStrengthAttrib();
	Tango::UserDefaultAttrProp	correctionstrength_prop;
	//	description	not set for CorrectionStrength
	correctionstrength_prop.set_label("correction strength");
	correctionstrength_prop.set_unit("m-3");
	//	standard_unit	not set for CorrectionStrength
	//	display_unit	not set for CorrectionStrength
	//	format	not set for CorrectionStrength
	//	max_value	not set for CorrectionStrength
	//	min_value	not set for CorrectionStrength
	//	max_alarm	not set for CorrectionStrength
	//	min_alarm	not set for CorrectionStrength
	//	max_warning	not set for CorrectionStrength
	//	min_warning	not set for CorrectionStrength
	//	delta_t	not set for CorrectionStrength
	//	delta_val	not set for CorrectionStrength
	correctionstrength->set_default_properties(correctionstrength_prop);
	//	Not Polled
	correctionstrength->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(correctionstrength);

	//	Attribute : ResonanceStrength
	ResonanceStrengthAttrib	*resonancestrength = new ResonanceStrengthAttrib();
	Tango::UserDefaultAttrProp	resonancestrength_prop;
	//	description	not set for ResonanceStrength
	resonancestrength_prop.set_label("resonance strength");
	resonancestrength_prop.set_unit("m-3");
	//	standard_unit	not set for ResonanceStrength
	//	display_unit	not set for ResonanceStrength
	//	format	not set for ResonanceStrength
	//	max_value	not set for ResonanceStrength
	//	min_value	not set for ResonanceStrength
	//	max_alarm	not set for ResonanceStrength
	//	min_alarm	not set for ResonanceStrength
	//	max_warning	not set for ResonanceStrength
	//	min_warning	not set for ResonanceStrength
	//	delta_t	not set for ResonanceStrength
	//	delta_val	not set for ResonanceStrength
	resonancestrength->set_default_properties(resonancestrength_prop);
	//	Not Polled
	resonancestrength->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(resonancestrength);

	//	Attribute : Current
	CurrentAttrib	*current = new CurrentAttrib();
	Tango::UserDefaultAttrProp	current_prop;
	//	description	not set for Current
	//	label	not set for Current
	current_prop.set_unit("A");
	//	standard_unit	not set for Current
	//	display_unit	not set for Current
	//	format	not set for Current
	//	max_value	not set for Current
	//	min_value	not set for Current
	//	max_alarm	not set for Current
	//	min_alarm	not set for Current
	//	max_warning	not set for Current
	//	min_warning	not set for Current
	//	delta_t	not set for Current
	//	delta_val	not set for Current
	current->set_default_properties(current_prop);
	//	Not Polled
	current->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(current);

	//	Attribute : BeamEnergy
	BeamEnergyAttrib	*beamenergy = new BeamEnergyAttrib();
	Tango::UserDefaultAttrProp	beamenergy_prop;
	beamenergy_prop.set_description("Electron beam energy in eV. Used to calculate magnetic rigidity in strength calculations.");
	beamenergy_prop.set_label("electron beam ebergy [eV]");
	//	unit	not set for BeamEnergy
	//	standard_unit	not set for BeamEnergy
	//	display_unit	not set for BeamEnergy
	//	format	not set for BeamEnergy
	//	max_value	not set for BeamEnergy
	//	min_value	not set for BeamEnergy
	//	max_alarm	not set for BeamEnergy
	//	min_alarm	not set for BeamEnergy
	//	max_warning	not set for BeamEnergy
	//	min_warning	not set for BeamEnergy
	//	delta_t	not set for BeamEnergy
	//	delta_val	not set for BeamEnergy
	beamenergy->set_default_properties(beamenergy_prop);
	//	Not Polled
	beamenergy->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(beamenergy);

	//	Attribute : DesignStrength
	DesignStrengthAttrib	*designstrength = new DesignStrengthAttrib();
	Tango::UserDefaultAttrProp	designstrength_prop;
	designstrength_prop.set_description("sextupole design strength 1/m^3 ( m-3). The strenght reqiured by the nominal optic\nThe attribute is memorized , expert only.");
	designstrength_prop.set_label("design strength");
	designstrength_prop.set_unit("m-3");
	//	standard_unit	not set for DesignStrength
	//	display_unit	not set for DesignStrength
	//	format	not set for DesignStrength
	//	max_value	not set for DesignStrength
	//	min_value	not set for DesignStrength
	//	max_alarm	not set for DesignStrength
	//	min_alarm	not set for DesignStrength
	//	max_warning	not set for DesignStrength
	//	min_warning	not set for DesignStrength
	//	delta_t	not set for DesignStrength
	//	delta_val	not set for DesignStrength
	designstrength->set_default_properties(designstrength_prop);
	//	Not Polled
	designstrength->set_disp_level(Tango::EXPERT);
	designstrength->set_memorized();
	designstrength->set_memorized_init(true);
	att_list.push_back(designstrength);

	//	Attribute : bbaEnable
	bbaEnableAttrib	*bbaenable = new bbaEnableAttrib();
	Tango::UserDefaultAttrProp	bbaenable_prop;
	bbaenable_prop.set_description("Read / and set the rele that modfy the sestupole cabling. If true the skew quadrupole QS is enabled, If fals the embedded horizontal corrector CEH is enabled.\nIf the magnet is permanetly cabled with the embedded skew quadrupole, the aiitrbute is always true nd is not writable.");
	bbaenable_prop.set_label("bba enabled");
	//	unit	not set for bbaEnable
	//	standard_unit	not set for bbaEnable
	//	display_unit	not set for bbaEnable
	//	format	not set for bbaEnable
	//	max_value	not set for bbaEnable
	//	min_value	not set for bbaEnable
	//	max_alarm	not set for bbaEnable
	//	min_alarm	not set for bbaEnable
	//	max_warning	not set for bbaEnable
	//	min_warning	not set for bbaEnable
	//	delta_t	not set for bbaEnable
	//	delta_val	not set for bbaEnable
	bbaenable->set_default_properties(bbaenable_prop);
	//	Not Polled
	bbaenable->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(bbaenable);

	//	Attribute : CEHStrength
	CEHStrengthAttrib	*cehstrength = new CEHStrengthAttrib();
	Tango::UserDefaultAttrProp	cehstrength_prop;
	cehstrength_prop.set_description("embbeded corrector horizontal kick");
	cehstrength_prop.set_label("horizontal kick");
	cehstrength_prop.set_unit("rad");
	//	standard_unit	not set for CEHStrength
	//	display_unit	not set for CEHStrength
	//	format	not set for CEHStrength
	//	max_value	not set for CEHStrength
	//	min_value	not set for CEHStrength
	//	max_alarm	not set for CEHStrength
	//	min_alarm	not set for CEHStrength
	//	max_warning	not set for CEHStrength
	//	min_warning	not set for CEHStrength
	//	delta_t	not set for CEHStrength
	//	delta_val	not set for CEHStrength
	cehstrength->set_default_properties(cehstrength_prop);
	//	Not Polled
	cehstrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(cehstrength);

	//	Attribute : CEHCorrectionStrength
	CEHCorrectionStrengthAttrib	*cehcorrectionstrength = new CEHCorrectionStrengthAttrib();
	Tango::UserDefaultAttrProp	cehcorrectionstrength_prop;
	//	description	not set for CEHCorrectionStrength
	cehcorrectionstrength_prop.set_label("horizontal emebedded corrector correction strength");
	cehcorrectionstrength_prop.set_unit("rad");
	//	standard_unit	not set for CEHCorrectionStrength
	//	display_unit	not set for CEHCorrectionStrength
	//	format	not set for CEHCorrectionStrength
	//	max_value	not set for CEHCorrectionStrength
	//	min_value	not set for CEHCorrectionStrength
	//	max_alarm	not set for CEHCorrectionStrength
	//	min_alarm	not set for CEHCorrectionStrength
	//	max_warning	not set for CEHCorrectionStrength
	//	min_warning	not set for CEHCorrectionStrength
	//	delta_t	not set for CEHCorrectionStrength
	//	delta_val	not set for CEHCorrectionStrength
	cehcorrectionstrength->set_default_properties(cehcorrectionstrength_prop);
	//	Not Polled
	cehcorrectionstrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(cehcorrectionstrength);

	//	Attribute : CEHResonanceStrength
	CEHResonanceStrengthAttrib	*cehresonancestrength = new CEHResonanceStrengthAttrib();
	Tango::UserDefaultAttrProp	cehresonancestrength_prop;
	//	description	not set for CEHResonanceStrength
	cehresonancestrength_prop.set_label("embedded horizontal corrector resonance strength");
	cehresonancestrength_prop.set_unit("rad");
	//	standard_unit	not set for CEHResonanceStrength
	//	display_unit	not set for CEHResonanceStrength
	//	format	not set for CEHResonanceStrength
	//	max_value	not set for CEHResonanceStrength
	//	min_value	not set for CEHResonanceStrength
	//	max_alarm	not set for CEHResonanceStrength
	//	min_alarm	not set for CEHResonanceStrength
	//	max_warning	not set for CEHResonanceStrength
	//	min_warning	not set for CEHResonanceStrength
	//	delta_t	not set for CEHResonanceStrength
	//	delta_val	not set for CEHResonanceStrength
	cehresonancestrength->set_default_properties(cehresonancestrength_prop);
	//	Not Polled
	cehresonancestrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(cehresonancestrength);

	//	Attribute : CEVStrength
	CEVStrengthAttrib	*cevstrength = new CEVStrengthAttrib();
	Tango::UserDefaultAttrProp	cevstrength_prop;
	cevstrength_prop.set_description("embbeded corrector vertical kick");
	cevstrength_prop.set_label("vertical kick");
	cevstrength_prop.set_unit("rad");
	//	standard_unit	not set for CEVStrength
	//	display_unit	not set for CEVStrength
	//	format	not set for CEVStrength
	//	max_value	not set for CEVStrength
	//	min_value	not set for CEVStrength
	//	max_alarm	not set for CEVStrength
	//	min_alarm	not set for CEVStrength
	//	max_warning	not set for CEVStrength
	//	min_warning	not set for CEVStrength
	//	delta_t	not set for CEVStrength
	//	delta_val	not set for CEVStrength
	cevstrength->set_default_properties(cevstrength_prop);
	//	Not Polled
	cevstrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(cevstrength);

	//	Attribute : CEVCorrectionStrength
	CEVCorrectionStrengthAttrib	*cevcorrectionstrength = new CEVCorrectionStrengthAttrib();
	Tango::UserDefaultAttrProp	cevcorrectionstrength_prop;
	//	description	not set for CEVCorrectionStrength
	cevcorrectionstrength_prop.set_label("vertical emebedded corrector correction strength");
	cevcorrectionstrength_prop.set_unit("rad");
	//	standard_unit	not set for CEVCorrectionStrength
	//	display_unit	not set for CEVCorrectionStrength
	//	format	not set for CEVCorrectionStrength
	//	max_value	not set for CEVCorrectionStrength
	//	min_value	not set for CEVCorrectionStrength
	//	max_alarm	not set for CEVCorrectionStrength
	//	min_alarm	not set for CEVCorrectionStrength
	//	max_warning	not set for CEVCorrectionStrength
	//	min_warning	not set for CEVCorrectionStrength
	//	delta_t	not set for CEVCorrectionStrength
	//	delta_val	not set for CEVCorrectionStrength
	cevcorrectionstrength->set_default_properties(cevcorrectionstrength_prop);
	//	Not Polled
	cevcorrectionstrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(cevcorrectionstrength);

	//	Attribute : CEVResonanceStrength
	CEVResonanceStrengthAttrib	*cevresonancestrength = new CEVResonanceStrengthAttrib();
	Tango::UserDefaultAttrProp	cevresonancestrength_prop;
	//	description	not set for CEVResonanceStrength
	cevresonancestrength_prop.set_label("embedded vertical corrector resonance strength");
	cevresonancestrength_prop.set_unit("rad");
	//	standard_unit	not set for CEVResonanceStrength
	//	display_unit	not set for CEVResonanceStrength
	//	format	not set for CEVResonanceStrength
	//	max_value	not set for CEVResonanceStrength
	//	min_value	not set for CEVResonanceStrength
	//	max_alarm	not set for CEVResonanceStrength
	//	min_alarm	not set for CEVResonanceStrength
	//	max_warning	not set for CEVResonanceStrength
	//	min_warning	not set for CEVResonanceStrength
	//	delta_t	not set for CEVResonanceStrength
	//	delta_val	not set for CEVResonanceStrength
	cevresonancestrength->set_default_properties(cevresonancestrength_prop);
	//	Not Polled
	cevresonancestrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(cevresonancestrength);

	//	Attribute : SQStrength
	SQStrengthAttrib	*sqstrength = new SQStrengthAttrib();
	Tango::UserDefaultAttrProp	sqstrength_prop;
	sqstrength_prop.set_description("embbeded skew quadrupole strength");
	sqstrength_prop.set_label("skew quad strength");
	sqstrength_prop.set_unit("1/m^2");
	//	standard_unit	not set for SQStrength
	//	display_unit	not set for SQStrength
	//	format	not set for SQStrength
	//	max_value	not set for SQStrength
	//	min_value	not set for SQStrength
	//	max_alarm	not set for SQStrength
	//	min_alarm	not set for SQStrength
	//	max_warning	not set for SQStrength
	//	min_warning	not set for SQStrength
	//	delta_t	not set for SQStrength
	//	delta_val	not set for SQStrength
	sqstrength->set_default_properties(sqstrength_prop);
	//	Not Polled
	sqstrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(sqstrength);

	//	Attribute : SQCorrectionStrength
	SQCorrectionStrengthAttrib	*sqcorrectionstrength = new SQCorrectionStrengthAttrib();
	Tango::UserDefaultAttrProp	sqcorrectionstrength_prop;
	//	description	not set for SQCorrectionStrength
	sqcorrectionstrength_prop.set_label("skew quadrupole correction strength");
	sqcorrectionstrength_prop.set_unit("1/m^2");
	//	standard_unit	not set for SQCorrectionStrength
	//	display_unit	not set for SQCorrectionStrength
	//	format	not set for SQCorrectionStrength
	//	max_value	not set for SQCorrectionStrength
	//	min_value	not set for SQCorrectionStrength
	//	max_alarm	not set for SQCorrectionStrength
	//	min_alarm	not set for SQCorrectionStrength
	//	max_warning	not set for SQCorrectionStrength
	//	min_warning	not set for SQCorrectionStrength
	//	delta_t	not set for SQCorrectionStrength
	//	delta_val	not set for SQCorrectionStrength
	sqcorrectionstrength->set_default_properties(sqcorrectionstrength_prop);
	//	Not Polled
	sqcorrectionstrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(sqcorrectionstrength);

	//	Attribute : SQResonanceStrength
	SQResonanceStrengthAttrib	*sqresonancestrength = new SQResonanceStrengthAttrib();
	Tango::UserDefaultAttrProp	sqresonancestrength_prop;
	//	description	not set for SQResonanceStrength
	sqresonancestrength_prop.set_label("skew quadrupole resonance strength");
	sqresonancestrength_prop.set_unit("1/m^2");
	//	standard_unit	not set for SQResonanceStrength
	//	display_unit	not set for SQResonanceStrength
	//	format	not set for SQResonanceStrength
	//	max_value	not set for SQResonanceStrength
	//	min_value	not set for SQResonanceStrength
	//	max_alarm	not set for SQResonanceStrength
	//	min_alarm	not set for SQResonanceStrength
	//	max_warning	not set for SQResonanceStrength
	//	min_warning	not set for SQResonanceStrength
	//	delta_t	not set for SQResonanceStrength
	//	delta_val	not set for SQResonanceStrength
	sqresonancestrength->set_default_properties(sqresonancestrength_prop);
	//	Not Polled
	sqresonancestrength->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(sqresonancestrength);

	//	Attribute : CEHCurrent
	CEHCurrentAttrib	*cehcurrent = new CEHCurrentAttrib();
	Tango::UserDefaultAttrProp	cehcurrent_prop;
	//	description	not set for CEHCurrent
	cehcurrent_prop.set_label("horizontal corrector current");
	cehcurrent_prop.set_unit("A");
	//	standard_unit	not set for CEHCurrent
	//	display_unit	not set for CEHCurrent
	//	format	not set for CEHCurrent
	//	max_value	not set for CEHCurrent
	//	min_value	not set for CEHCurrent
	//	max_alarm	not set for CEHCurrent
	//	min_alarm	not set for CEHCurrent
	//	max_warning	not set for CEHCurrent
	//	min_warning	not set for CEHCurrent
	//	delta_t	not set for CEHCurrent
	//	delta_val	not set for CEHCurrent
	cehcurrent->set_default_properties(cehcurrent_prop);
	//	Not Polled
	cehcurrent->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(cehcurrent);

	//	Attribute : CEVCurrent
	CEVCurrentAttrib	*cevcurrent = new CEVCurrentAttrib();
	Tango::UserDefaultAttrProp	cevcurrent_prop;
	//	description	not set for CEVCurrent
	cevcurrent_prop.set_label("vertical corrector current");
	cevcurrent_prop.set_unit("A");
	//	standard_unit	not set for CEVCurrent
	//	display_unit	not set for CEVCurrent
	//	format	not set for CEVCurrent
	//	max_value	not set for CEVCurrent
	//	min_value	not set for CEVCurrent
	//	max_alarm	not set for CEVCurrent
	//	min_alarm	not set for CEVCurrent
	//	max_warning	not set for CEVCurrent
	//	min_warning	not set for CEVCurrent
	//	delta_t	not set for CEVCurrent
	//	delta_val	not set for CEVCurrent
	cevcurrent->set_default_properties(cevcurrent_prop);
	//	Not Polled
	cevcurrent->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(cevcurrent);

	//	Attribute : PolynomA
	PolynomAAttrib	*polynoma = new PolynomAAttrib();
	Tango::UserDefaultAttrProp	polynoma_prop;
	polynoma_prop.set_description("array of normal multipole components");
	polynoma_prop.set_label("normal multipoles");
	//	unit	not set for PolynomA
	//	standard_unit	not set for PolynomA
	//	display_unit	not set for PolynomA
	//	format	not set for PolynomA
	//	max_value	not set for PolynomA
	//	min_value	not set for PolynomA
	//	max_alarm	not set for PolynomA
	//	min_alarm	not set for PolynomA
	//	max_warning	not set for PolynomA
	//	min_warning	not set for PolynomA
	//	delta_t	not set for PolynomA
	//	delta_val	not set for PolynomA
	polynoma->set_default_properties(polynoma_prop);
	//	Not Polled
	polynoma->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(polynoma);

	//	Attribute : PolynomB
	PolynomBAttrib	*polynomb = new PolynomBAttrib();
	Tango::UserDefaultAttrProp	polynomb_prop;
	polynomb_prop.set_description("array of skew multipole components");
	polynomb_prop.set_label("skew multipoles");
	//	unit	not set for PolynomB
	//	standard_unit	not set for PolynomB
	//	display_unit	not set for PolynomB
	//	format	not set for PolynomB
	//	max_value	not set for PolynomB
	//	min_value	not set for PolynomB
	//	max_alarm	not set for PolynomB
	//	min_alarm	not set for PolynomB
	//	max_warning	not set for PolynomB
	//	min_warning	not set for PolynomB
	//	delta_t	not set for PolynomB
	//	delta_val	not set for PolynomB
	polynomb->set_default_properties(polynomb_prop);
	//	Not Polled
	polynomb->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(polynomb);


	//	Create a list of static attributes
	create_static_attribute_list(get_class_attr()->get_attr_list());
	/*----- PROTECTED REGION ID(E2SextupoleClass::attribute_factory_after) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::attribute_factory_after
}
//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::pipe_factory()
 *	Description: Create the pipe object(s)
 *                and store them in the pipe list
 */
//--------------------------------------------------------
void E2SextupoleClass::pipe_factory()
{
	/*----- PROTECTED REGION ID(E2SextupoleClass::pipe_factory_before) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::pipe_factory_before
	/*----- PROTECTED REGION ID(E2SextupoleClass::pipe_factory_after) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::pipe_factory_after
}
//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::command_factory()
 *	Description: Create the command object(s)
 *                and store them in the command list
 */
//--------------------------------------------------------
void E2SextupoleClass::command_factory()
{
	/*----- PROTECTED REGION ID(E2SextupoleClass::command_factory_before) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::command_factory_before


	//	Command Reset
	ResetClass	*pResetCmd =
		new ResetClass("Reset",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pResetCmd);

	//	Command ResetResonanceStrength
	ResetResonanceStrengthClass	*pResetResonanceStrengthCmd =
		new ResetResonanceStrengthClass("ResetResonanceStrength",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pResetResonanceStrengthCmd);

	//	Command ApplyDesignStrength
	ApplyDesignStrengthClass	*pApplyDesignStrengthCmd =
		new ApplyDesignStrengthClass("ApplyDesignStrength",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pApplyDesignStrengthCmd);

	/*----- PROTECTED REGION ID(E2SextupoleClass::command_factory_after) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::command_factory_after
}

//===================================================================
//	Dynamic attributes related methods
//===================================================================

//--------------------------------------------------------
/**
 * method : 		E2SextupoleClass::create_static_attribute_list
 * description : 	Create the a list of static attributes
 *
 * @param	att_list	the created attribute list
 */
//--------------------------------------------------------
void E2SextupoleClass::create_static_attribute_list(std::vector<Tango::Attr *> &att_list)
{
	for (unsigned long i=0 ; i<att_list.size() ; i++)
	{
		std::string att_name(att_list[i]->get_name());
		std::transform(att_name.begin(), att_name.end(), att_name.begin(), ::tolower);
		defaultAttList.push_back(att_name);
	}

	TANGO_LOG_INFO << defaultAttList.size() << " attributes in default list" << std::endl;

	/*----- PROTECTED REGION ID(E2SextupoleClass::create_static_att_list) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::create_static_att_list
}


//--------------------------------------------------------
/**
 * method : 		E2SextupoleClass::erase_dynamic_attributes
 * description : 	delete the dynamic attributes if any.
 *
 * @param	devlist_ptr	the device list pointer
 * @param	list of all attributes
 */
//--------------------------------------------------------
void E2SextupoleClass::erase_dynamic_attributes(const Tango::DevVarStringArray *devlist_ptr, std::vector<Tango::Attr *> &att_list)
{
	Tango::Util *tg = Tango::Util::instance();

	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		Tango::DeviceImpl *dev_impl = tg->get_device_by_name(((std::string)(*devlist_ptr)[i]).c_str());
		E2Sextupole *dev = static_cast<E2Sextupole *> (dev_impl);

		std::vector<Tango::Attribute *> &dev_att_list = dev->get_device_attr()->get_attribute_list();
		std::vector<Tango::Attribute *>::iterator ite_att;
		for (ite_att=dev_att_list.begin() ; ite_att != dev_att_list.end() ; ++ite_att)
		{
			std::string att_name((*ite_att)->get_name_lower());
			if ((att_name == "state") || (att_name == "status"))
				continue;
			std::vector<std::string>::iterator ite_str = find(defaultAttList.begin(), defaultAttList.end(), att_name);
			if (ite_str == defaultAttList.end())
			{
				TANGO_LOG_INFO << att_name << " is a UNWANTED dynamic attribute for device " << (*devlist_ptr)[i] << std::endl;
				Tango::Attribute &att = dev->get_device_attr()->get_attr_by_name(att_name.c_str());
				dev->remove_attribute(att_list[att.get_attr_idx()], true, false);
				--ite_att;
			}
		}
	}
	/*----- PROTECTED REGION ID(E2SextupoleClass::erase_dynamic_attributes) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::erase_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method     : E2SextupoleClass::get_attr_object_by_name()
 *	Description: returns Tango::Attr * object found by name
 */
//--------------------------------------------------------
Tango::Attr *E2SextupoleClass::get_attr_object_by_name(std::vector<Tango::Attr *> &att_list, std::string attname)
{
	std::vector<Tango::Attr *>::iterator it;
	for (it=att_list.begin() ; it<att_list.end() ; ++it)
		if ((*it)->get_name()==attname)
			return (*it);
	//	Attr does not exist
	return NULL;
}


/*----- PROTECTED REGION ID(E2SextupoleClass::Additional Methods) ENABLED START -----*/
/* clang-format on */
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2SextupoleClass::Additional Methods
} //	namespace
